@@ -19,6 +19,7 @@
  * Boston, MA 02111-1307, USA.
  *
  * Author: JP Rosevear
+ * Author: Naba Kumar
  */
 
 #ifdef HAVE_CONFIG_H
@@ -43,6 +44,7 @@
 #include "gbf-am-project.h"
 #include "gbf-am-build.h"
 #include "gbf-am-config.h"
+#include "gbf-am-properties.h"
 
 #define ENABLE_DEBUG
 
@@ -543,9 +545,9 @@ xml_new_source_node (GbfAmProject *project,
 	xmlNodePtr source;
 	gchar *filename;
 	
-	source = xmlNewDocNode (doc, NULL, "source", NULL);
+	source = xmlNewDocNode (doc, NULL, BAD_CAST "source", NULL);
 	filename = uri_get_chrooted_path (project->project_root_uri, uri);
-	xmlSetProp (source, "uri", filename);
+	xmlSetProp (source, BAD_CAST "uri", BAD_CAST filename);
 	g_free (filename);
 	
 	return source;
@@ -567,17 +569,17 @@ xml_write_location_recursive (GbfAmProject *project,
 		GbfAmNode *node = GBF_AM_NODE (g_node);
 		switch (node->type) {
 			case GBF_AM_NODE_GROUP:
-				xml_node = xmlNewDocNode (doc, NULL, "group", NULL);
-				xmlSetProp (xml_node, "id", node->id);
+				xml_node = xmlNewDocNode (doc, NULL, BAD_CAST "group", NULL);
+				xmlSetProp (xml_node, BAD_CAST "id", BAD_CAST node->id);
 				finished = TRUE;
 				break;
 			case GBF_AM_NODE_TARGET:
-				xml_node = xmlNewDocNode (doc, NULL, "target", NULL);
+				xml_node = xmlNewDocNode (doc, NULL, BAD_CAST "target", NULL);
 				/* strip the group id from the target
 				   id, since the script identifies
 				   targets only within the group */
-				xmlSetProp (xml_node, "id",
-					    node->id +
+				xmlSetProp (xml_node, BAD_CAST "id",
+					    BAD_CAST node->id +
 					    strlen (GBF_AM_NODE (g_node->parent)->id));
 				break;
 			case GBF_AM_NODE_SOURCE:
@@ -613,8 +615,8 @@ xml_write_add_source (GbfAmProject *project,
 {
 	xmlNodePtr cur, source;
 	
-	cur = xmlNewDocNode (doc, NULL, "add", NULL);
-	xmlSetProp (cur, "type", "source");
+	cur = xmlNewDocNode (doc, NULL, BAD_CAST "add", NULL);
+	xmlSetProp (cur, BAD_CAST "type", BAD_CAST "source");
 	xmlAddChild (doc->children, cur);
 
 	cur = xml_write_location_recursive (project, doc, cur, g_node);
@@ -631,8 +633,8 @@ xml_write_remove_source (GbfAmProject *project,
 {
 	xmlNodePtr cur;
 
-	cur = xmlNewDocNode (doc, NULL, "remove", NULL);
-	xmlSetProp (cur, "type", "source");
+	cur = xmlNewDocNode (doc, NULL, BAD_CAST "remove", NULL);
+	xmlSetProp (cur, BAD_CAST "type", BAD_CAST "source");
 	xmlAddChild (doc->children, cur);
 	
 	cur = xml_write_location_recursive (project, doc, cur, g_node);
@@ -651,20 +653,171 @@ xml_write_add_target (GbfAmProject *project,
 	
 	g_assert (GBF_AM_NODE (g_node)->type == GBF_AM_NODE_GROUP);
 
-	cur = xmlNewDocNode (doc, NULL, "add", NULL);
-	xmlSetProp (cur, "type", "target");
+	cur = xmlNewDocNode (doc, NULL, BAD_CAST "add", NULL);
+	xmlSetProp (cur, BAD_CAST "type", BAD_CAST "target");
 	xmlAddChild (doc->children, cur);
 
 	cur = xml_write_location_recursive (project, doc, cur, g_node);
 	
-	target = xmlNewDocNode (doc, NULL, "target", NULL);
-	xmlSetProp (target, "id", name);
-	xmlSetProp (target, "type", type);
+	target = xmlNewDocNode (doc, NULL, BAD_CAST "target", NULL);
+	xmlSetProp (target, BAD_CAST "id", BAD_CAST name);
+	xmlSetProp (target, BAD_CAST "type", BAD_CAST type);
 	xmlAddChild (cur, target);
 
 	return TRUE;
 }
 
+typedef struct {
+	GbfAmConfigMapping *old_config;
+	xmlDocPtr doc;
+	xmlNodePtr curr_xml_node;
+} GbfXmlWriteData;
+
+static void
+xml_write_set_item_config_cb (const gchar *param, GbfAmConfigValue *value,
+			      gpointer user_data)
+{
+	xmlNodePtr param_node;
+	GbfXmlWriteData *data = (GbfXmlWriteData*)user_data;
+	
+	if (value->type == GBF_AM_TYPE_STRING) {
+		GbfAmConfigValue *old_value = NULL;
+		const gchar *old_str = "", *new_str = "";
+		
+		new_str = gbf_am_config_value_get_string (value);
+		if (new_str == NULL)
+			new_str = "";
+		if (data->old_config)
+			old_value = gbf_am_config_mapping_lookup (data->old_config, param);
+		if (old_value) {
+			old_str = gbf_am_config_value_get_string (old_value);
+			if (old_str == NULL)
+				old_str = "";
+		}
+		if (strcmp (new_str, old_str) != 0)
+		{
+			param_node = xmlNewDocNode (data->doc, NULL,
+						    BAD_CAST "item", NULL);
+			xmlSetProp (param_node, BAD_CAST "name", BAD_CAST param);
+			xmlSetProp (param_node, BAD_CAST "value", BAD_CAST new_str);
+			xmlAddChild (data->curr_xml_node, param_node);
+		}
+	}
+}
+
+static void
+xml_write_set_param_config_cb (const gchar *param, GbfAmConfigValue *value,
+			       gpointer user_data)
+{
+	xmlNodePtr param_node;
+	GbfXmlWriteData *data = (GbfXmlWriteData*)user_data;
+	
+	if (value->type == GBF_AM_TYPE_STRING) {
+		GbfAmConfigValue *old_value;
+		const gchar *old_str = "", *new_str = "";
+		
+		new_str = gbf_am_config_value_get_string (value);
+		if (new_str == NULL)
+			new_str = "";
+		
+		old_value = gbf_am_config_mapping_lookup (data->old_config, param);
+		if (old_value) {
+			old_str = gbf_am_config_value_get_string (old_value);
+			if (old_str == NULL)
+				old_str = "";
+		}
+		if (strcmp (new_str, old_str) != 0)
+		{
+			param_node = xmlNewDocNode (data->doc, NULL,
+						    BAD_CAST "param", NULL);
+			xmlSetProp (param_node, BAD_CAST "name", BAD_CAST param);
+			xmlSetProp (param_node, BAD_CAST "value", BAD_CAST new_str);
+			xmlAddChild (data->curr_xml_node, param_node);
+		}
+	} else if (value->type == GBF_AM_TYPE_LIST) {
+		param_node = xmlNewDocNode (data->doc, NULL,
+					    BAD_CAST "param", NULL);
+		xmlSetProp (param_node, BAD_CAST "name", BAD_CAST param);
+		/* FIXME */
+	} else if (value->type == GBF_AM_TYPE_MAPPING) {
+		
+		GbfXmlWriteData write_data;
+		GbfAmConfigValue *old_value = NULL;
+		GbfAmConfigMapping *new_mapping = NULL, *old_mapping = NULL;
+		
+		new_mapping = gbf_am_config_value_get_mapping (value);
+		old_value = gbf_am_config_mapping_lookup (data->old_config, param);
+		if (old_value)
+			old_mapping = gbf_am_config_value_get_mapping (old_value);
+		
+		param_node = xmlNewDocNode (data->doc, NULL, BAD_CAST "param", NULL);
+		
+		xmlSetProp (param_node, BAD_CAST "name", BAD_CAST param);
+		
+		write_data.doc = data->doc;
+		write_data.curr_xml_node = param_node;
+		write_data.old_config = old_mapping;
+		
+		gbf_am_config_mapping_foreach (new_mapping,
+					       xml_write_set_item_config_cb,
+					       &write_data);
+		if (param_node->children)
+			xmlAddChild (data->curr_xml_node, param_node);
+		else
+			xmlFreeNode (param_node);
+	} else {
+		g_warning ("Should not be here");
+	}
+}
+
+static gboolean 
+xml_write_set_config (GbfAmProject       *project,
+		      xmlDocPtr           doc,
+		      GNode              *g_node,
+		      GbfAmConfigMapping *new_config)
+{
+	xmlNodePtr cur, config;
+	GbfXmlWriteData user_data;
+	
+	cur = xmlNewDocNode (doc, NULL, BAD_CAST "set", NULL);
+	xmlSetProp (cur, BAD_CAST "type", BAD_CAST "config");
+	xmlAddChild (doc->children, cur);
+
+	if (g_node)
+		cur = xml_write_location_recursive (project, doc, cur, g_node);
+	
+	config = xmlNewDocNode (doc, NULL, BAD_CAST "config", NULL);
+	xmlAddChild (cur, config);
+	
+	user_data.doc = doc;
+	user_data.curr_xml_node = config;
+	user_data.old_config = g_node? GBF_AM_NODE (g_node)->config : project->project_config;
+	
+	gbf_am_config_mapping_foreach (new_config,
+				       xml_write_set_param_config_cb,
+				       &user_data);
+	if (config->children)
+		return TRUE;
+	else
+		return FALSE;
+}
+
+static gboolean
+xml_write_remove_target (GbfAmProject *project,
+			 xmlDocPtr     doc,
+			 GNode        *g_node)
+{
+	xmlNodePtr cur;
+
+	cur = xmlNewDocNode (doc, NULL, BAD_CAST "remove", NULL);
+	xmlSetProp (cur, BAD_CAST "type", BAD_CAST "target");
+	xmlAddChild (doc->children, cur);
+	
+	cur = xml_write_location_recursive (project, doc, cur, g_node);
+	
+	return (cur != NULL);
+}
+
 static gboolean 
 xml_write_add_group (GbfAmProject *project,
 		     xmlDocPtr     doc,
@@ -676,32 +829,48 @@ xml_write_add_group (GbfAmProject *project,
 	
 	g_assert (GBF_AM_NODE (g_node)->type == GBF_AM_NODE_GROUP);
 
-	cur = xmlNewDocNode (doc, NULL, "add", NULL);
-	xmlSetProp (cur, "type", "group");
+	cur = xmlNewDocNode (doc, NULL, BAD_CAST "add", NULL);
+	xmlSetProp (cur, BAD_CAST "type", BAD_CAST "group");
 	xmlAddChild (doc->children, cur);
 
 	/* calculate new id needed by the script */
 	new_id = g_strdup_printf ("%s%s/", GBF_AM_NODE (g_node)->id, new_group);
 	
-	group = xmlNewDocNode (doc, NULL, "group", NULL);
-	xmlSetProp (group, "id", new_id);
+	group = xmlNewDocNode (doc, NULL, BAD_CAST "group", NULL);
+	xmlSetProp (group, BAD_CAST "id", BAD_CAST new_id);
 	xmlAddChild (cur, group);
 	g_free (new_id);
 	
 	return TRUE;
 }
 
+static gboolean
+xml_write_remove_group (GbfAmProject *project,
+			 xmlDocPtr     doc,
+			 GNode        *g_node)
+{
+	xmlNodePtr cur;
+
+	cur = xmlNewDocNode (doc, NULL, BAD_CAST "remove", NULL);
+	xmlSetProp (cur, BAD_CAST "type", BAD_CAST "group");
+	xmlAddChild (doc->children, cur);
+	
+	cur = xml_write_location_recursive (project, doc, cur, g_node);
+	
+	return (cur != NULL);
+}
+
 static xmlDocPtr
 xml_new_change_doc (GbfAmProject *project)
 {
 	xmlDocPtr doc;
 	
-	doc = xmlNewDoc ("1.0");
+	doc = xmlNewDoc (BAD_CAST "1.0");
 	if (doc != NULL) {
 		gchar *root_path;
 		root_path = uri_to_path (project->project_root_uri);
-		doc->children = xmlNewDocNode (doc, NULL, "project", NULL);
-		xmlSetProp (doc->children, "root", root_path);
+		doc->children = xmlNewDocNode (doc, NULL, BAD_CAST "project", NULL);
+		xmlSetProp (doc->children, BAD_CAST "root", BAD_CAST root_path);
 		g_free (root_path);
 	}
 
@@ -964,7 +1133,7 @@ sax_start_element (void *ctxt, const xmlChar *name, const xmlChar **attrs)
 		/* assign here, since we destroy the data in project_data_init() */
 		if (project_file) {
 			g_free (project->project_file);
-			project->project_file = g_strdup (project_file);
+			project->project_file = g_strdup ((char *) project_file);
 		}
 		
 	} else if (xmlStrEqual (name, BAD_CAST "group")) {
@@ -1000,7 +1169,7 @@ sax_start_element (void *ctxt, const xmlChar *name, const xmlChar **attrs)
 			g_node = project_node_new (GBF_AM_NODE_GROUP);
 			node = GBF_AM_NODE (g_node);
 			
-			node->id = g_strdup (group_id);
+			node->id = g_strdup ((char *) group_id);
 			
 			/* set parent group */
 			if (data->current_node != NULL) {
@@ -1048,8 +1217,8 @@ sax_start_element (void *ctxt, const xmlChar *name, const xmlChar **attrs)
 				child = g_node_next_sibling (child);
 			}
 		}
-		node->name = g_strdup (group_name);
-		node->uri = g_strdup (group_source);
+		node->name = g_strdup ((char *) group_name);
+		node->uri = g_strdup ((char *) group_source);
 		node->config = gbf_am_config_mapping_new ();
 		
 		/* set working node */
@@ -1129,8 +1298,8 @@ sax_start_element (void *ctxt, const xmlChar *name, const xmlChar **attrs)
 				child = g_node_next_sibling (child);
 			}
 		}
-		node->name = g_strdup (target_name);
-		node->detail = g_strdup (target_type);
+		node->name = g_strdup ((char *) target_name);
+		node->detail = g_strdup ((char *) target_type);
 		node->config = gbf_am_config_mapping_new ();
 
 		/* set working node */
@@ -1255,7 +1424,7 @@ sax_start_element (void *ctxt, const xmlChar *name, const xmlChar **attrs)
 			g_free (node->detail);
 		}
 		node->uri = source_uri;
-		node->detail = g_strdup (target_dep);
+		node->detail = g_strdup ((char *) target_dep);
 
 		/* set working node */
 		data->current_node = g_node;
@@ -1309,11 +1478,11 @@ sax_start_element (void *ctxt, const xmlChar *name, const xmlChar **attrs)
 
 			/* save scalar string parameter */
 			param = gbf_am_config_value_new (GBF_AM_TYPE_STRING);
-			gbf_am_config_value_set_string (param, param_value);
+			gbf_am_config_value_set_string (param, (char *) param_value);
 
 			/* try to save the parameter in the config mapping */
 			if (!gbf_am_config_mapping_insert (data->config,
-							   param_name, param)) {
+							   (char *) param_name, param)) {
 				gbf_am_config_value_free (param);
 			}
 
@@ -1322,7 +1491,7 @@ sax_start_element (void *ctxt, const xmlChar *name, const xmlChar **attrs)
 
 		} else {
 			/* we expect a list or a hash of values */
-			data->param_key = g_strdup (param_name);
+			data->param_key = g_strdup ((char *) param_name);
 			data->state = PARSE_PARAM;
 		}
 		
@@ -1352,7 +1521,7 @@ sax_start_element (void *ctxt, const xmlChar *name, const xmlChar **attrs)
 		
 		/* create item value */
 		item = gbf_am_config_value_new (GBF_AM_TYPE_STRING);
-		gbf_am_config_value_set_string (item, item_value);
+		gbf_am_config_value_set_string (item, (char *) item_value);
 		
 		/* get current configuration parameter */
 		param = gbf_am_config_mapping_lookup (data->config, data->param_key);
@@ -1380,7 +1549,7 @@ sax_start_element (void *ctxt, const xmlChar *name, const xmlChar **attrs)
 		    case GBF_AM_TYPE_MAPPING:
 			    if (item_name == NULL ||
 				!gbf_am_config_mapping_insert (param->mapping,
-							       item_name,
+							       (char *) item_name,
 							       item)) {
 				    gbf_am_config_value_free (item);
 			    }
@@ -2102,7 +2271,7 @@ project_update (GbfAmProject *project,
 	gboolean retval;
 	gint i;
 	xmlChar *xml_doc;
-	int xml_size;
+	gint xml_size;
 	
 	/* remove file monitors */
 	monitors_remove (project);
@@ -2121,7 +2290,7 @@ project_update (GbfAmProject *project,
 
 	/* execute the script */
 	data = spawn_script (argv, SCRIPT_TIMEOUT,
-			     xml_doc, xml_size,  /* input buffer */
+			     (char *) xml_doc, xml_size,  /* input buffer */
 			     NULL, NULL, NULL);  /* i/o callbacks */
 	xmlFree (xml_doc);
 
@@ -2269,6 +2438,177 @@ project_data_init (GbfAmProject *project)
 	project->sources = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 }
 
+GbfAmConfigMapping *
+gbf_am_project_get_config (GbfAmProject *project, GError **error)
+{
+	g_return_val_if_fail (GBF_IS_AM_PROJECT (project), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+	
+	return gbf_am_config_mapping_copy (project->project_config);
+}
+
+GbfAmConfigMapping *
+gbf_am_project_get_group_config (GbfAmProject *project, const gchar *group_id,
+				 GError **error)
+{
+	GbfAmNode *node;
+	GNode *g_node;
+	
+	g_return_val_if_fail (GBF_IS_AM_PROJECT (project), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+	
+	g_node = g_hash_table_lookup (project->groups, group_id);
+	if (g_node == NULL) {
+		error_set (error, GBF_PROJECT_ERROR_DOESNT_EXIST,
+			   _("Group doesn't exist"));
+		return NULL;
+	}
+	node = GBF_AM_NODE (g_node);
+	return gbf_am_config_mapping_copy (node->config);
+}
+
+GbfAmConfigMapping *
+gbf_am_project_get_target_config (GbfAmProject *project, const gchar *target_id,
+				  GError **error)
+{
+	GbfAmNode *node;
+	GNode *g_node;
+	
+	g_return_val_if_fail (GBF_IS_AM_PROJECT (project), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+	
+	g_node = g_hash_table_lookup (project->targets, target_id);
+	if (g_node == NULL) {
+		error_set (error, GBF_PROJECT_ERROR_DOESNT_EXIST,
+			   _("Target doesn't exist"));
+		return NULL;
+	}
+	node = GBF_AM_NODE (g_node);
+	return gbf_am_config_mapping_copy (node->config);
+}
+
+void
+gbf_am_project_set_config (GbfAmProject *project,
+			   GbfAmConfigMapping *new_config, GError **error)
+{
+	xmlDocPtr doc;
+	GSList *change_set = NULL;
+	
+	g_return_if_fail (GBF_IS_AM_PROJECT (project));
+	g_return_if_fail (new_config != NULL);
+	g_return_if_fail (error == NULL || *error == NULL);
+	
+	/* Create the update xml */
+	doc = xml_new_change_doc (project);
+	
+	if (!xml_write_set_config (project, doc, NULL, new_config)) {
+		xmlFreeDoc (doc);
+		return;
+	}
+
+	DEBUG ({
+		xmlSetDocCompressMode (doc, 0);
+		xmlSaveFile ("/tmp/set-config.xml", doc);
+	});
+
+	/* Update the project */
+	if (!project_update (project, doc, &change_set, error)) {
+		error_set (error, GBF_PROJECT_ERROR_PROJECT_MALFORMED,
+			   _("Unable to update project"));
+		xmlFreeDoc (doc);
+		return;
+	}
+	xmlFreeDoc (doc);
+	change_set_destroy (change_set);
+}
+
+void
+gbf_am_project_set_group_config (GbfAmProject *project, const gchar *group_id,
+				 GbfAmConfigMapping *new_config, GError **error)
+{
+	GbfAmNode *node;
+	xmlDocPtr doc;
+	GNode *g_node;
+	GSList *change_set = NULL;
+	
+	g_return_if_fail (GBF_IS_AM_PROJECT (project));
+	g_return_if_fail (new_config != NULL);
+	g_return_if_fail (error == NULL || *error == NULL);
+	
+	g_node = g_hash_table_lookup (project->groups, group_id);
+	if (g_node == NULL) {
+		error_set (error, GBF_PROJECT_ERROR_DOESNT_EXIST,
+			   _("Group doesn't exist"));
+		return;
+	}
+	node = GBF_AM_NODE (g_node);
+	
+	/* Create the update xml */
+	doc = xml_new_change_doc (project);
+	if (!xml_write_set_config (project, doc, g_node, new_config)) {
+		xmlFreeDoc (doc);
+		return;
+	}
+
+	DEBUG ({
+		xmlSetDocCompressMode (doc, 0);
+		xmlSaveFile ("/tmp/set-config.xml", doc);
+	});
+	
+	/* Update the project */
+	if (!project_update (project, doc, &change_set, error)) {
+		error_set (error, GBF_PROJECT_ERROR_PROJECT_MALFORMED,
+			   _("Unable to update project"));
+		xmlFreeDoc (doc);
+		return;
+	}
+	xmlFreeDoc (doc);
+	change_set_destroy (change_set);
+}
+
+void
+gbf_am_project_set_target_config (GbfAmProject *project,
+				  const gchar *target_id,
+				  GbfAmConfigMapping *new_config,
+				  GError **error)
+{
+	xmlDocPtr doc;
+	GNode *g_node;
+	GSList *change_set = NULL;
+	
+	g_return_if_fail (GBF_IS_AM_PROJECT (project));
+	g_return_if_fail (new_config != NULL);
+	g_return_if_fail (error == NULL || *error == NULL);
+	
+	g_node = g_hash_table_lookup (project->targets, target_id);
+	if (g_node == NULL) {
+		error_set (error, GBF_PROJECT_ERROR_DOESNT_EXIST,
+			   _("Target doesn't exist"));
+	}
+	
+	/* Create the update xml */
+	doc = xml_new_change_doc (project);
+	if (!xml_write_set_config (project, doc, g_node, new_config)) {
+		xmlFreeDoc (doc);
+		return;
+	}
+
+	DEBUG ({
+		xmlSetDocCompressMode (doc, 0);
+		xmlSaveFile ("/tmp/set-config.xml", doc);
+	});
+
+	/* Update the project */
+	if (!project_update (project, doc, &change_set, error)) {
+		error_set (error, GBF_PROJECT_ERROR_PROJECT_MALFORMED,
+			   _("Unable to update project"));
+		xmlFreeDoc (doc);
+		return;
+	}
+	xmlFreeDoc (doc);
+	change_set_destroy (change_set);
+}
+
 /*
  * GbfProjectIface methods ------------------------------------------
  */
@@ -2580,6 +2920,30 @@ impl_get_group (GbfProject  *_project,
 	return group;
 }
 
+static void
+foreach_group (gpointer key, gpointer value, gpointer data)
+{
+	GList **groups = data;
+
+	*groups = g_list_prepend (*groups, g_strdup (key));
+}
+
+
+static GList *
+impl_get_all_groups (GbfProject *_project,
+		     GError    **error)
+{
+	GbfAmProject *project;
+	GList *groups = NULL;
+
+	g_return_val_if_fail (GBF_IS_AM_PROJECT (_project), NULL);
+
+	project = GBF_AM_PROJECT (_project);
+	g_hash_table_foreach (project->groups, foreach_group, &groups);
+
+	return groups;
+}
+
 static GtkWidget *
 impl_configure_new_group (GbfProject *_project,
 			  GError    **error)
@@ -2594,9 +2958,19 @@ impl_configure_group (GbfProject  *_project,
 		      const gchar *id,
 		      GError     **error)
 {
-	UNIMPLEMENTED;
-
-	return NULL;
+	GtkWidget *wid = NULL;
+	GError *err = NULL;
+	
+	g_return_val_if_fail (GBF_IS_PROJECT (_project), NULL);
+	g_return_val_if_fail (id != NULL, NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+	
+	wid = gbf_am_properties_get_group_widget (GBF_AM_PROJECT (_project),
+						  id, &err);
+	if (err) {
+		g_propagate_error (error, err);
+	}
+	return wid;
 }
 
 static gchar * 
@@ -2680,7 +3054,44 @@ impl_remove_group (GbfProject  *_project,
 		   const gchar *id,
 		   GError     **error)
 {
-	UNIMPLEMENTED;
+	GbfAmProject *project;
+	GNode *g_node;
+	xmlDocPtr doc;
+	GSList *change_set = NULL;
+	
+	g_return_if_fail (GBF_IS_AM_PROJECT (_project));
+
+	project = GBF_AM_PROJECT (_project);
+	
+	/* Find the target */
+	g_node = g_hash_table_lookup (project->groups, id);
+	if (g_node == NULL) {
+		error_set (error, GBF_PROJECT_ERROR_DOESNT_EXIST,
+			   _("Group doesn't exist"));
+		return;
+	}
+
+	/* Create the update xml */
+	doc = xml_new_change_doc (project);
+	if (!xml_write_remove_group (project, doc, g_node)) {
+		error_set (error, GBF_PROJECT_ERROR_DOESNT_EXIST,
+			   _("Group couldn't be removed"));
+		xmlFreeDoc (doc);
+		return;
+	}
+
+	DEBUG ({
+		xmlSetDocCompressMode (doc, 0);
+		xmlSaveFile ("/tmp/remove-group.xml", doc);
+	});
+
+	/* Update the project */
+	if (!project_update (project, doc, &change_set, error)) {
+		error_set (error, GBF_PROJECT_ERROR_PROJECT_MALFORMED,
+			   _("Unable to update project"));
+	}
+	xmlFreeDoc (doc);
+	change_set_destroy (change_set);
 }
 
 static GbfProjectTarget * 
@@ -2738,7 +3149,6 @@ foreach_target (gpointer key, gpointer value, gpointer data)
 	*targets = g_list_prepend (*targets, g_strdup (key));
 }
 
-
 static GList *
 impl_get_all_targets (GbfProject *_project,
 		      GError    **error)
@@ -2768,9 +3178,19 @@ impl_configure_target (GbfProject  *_project,
 		       const gchar *id,
 		       GError     **error)
 {
-	UNIMPLEMENTED;
-
-	return NULL;
+	GtkWidget *wid = NULL;
+	GError *err = NULL;
+	
+	g_return_val_if_fail (GBF_IS_PROJECT (_project), NULL);
+	g_return_val_if_fail (id != NULL, NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+	
+	wid = gbf_am_properties_get_target_widget (GBF_AM_PROJECT (_project),
+						   id, &err);
+	if (err) {
+		g_propagate_error (error, err);
+	}
+	return wid;
 }
 
 static char * 
@@ -2857,7 +3277,44 @@ impl_remove_target (GbfProject  *_project,
 		    const gchar *id,
 		    GError     **error)
 {
-	UNIMPLEMENTED;
+	GbfAmProject *project;
+	GNode *g_node;
+	xmlDocPtr doc;
+	GSList *change_set = NULL;
+	
+	g_return_if_fail (GBF_IS_AM_PROJECT (_project));
+
+	project = GBF_AM_PROJECT (_project);
+	
+	/* Find the target */
+	g_node = g_hash_table_lookup (project->targets, id);
+	if (g_node == NULL) {
+		error_set (error, GBF_PROJECT_ERROR_DOESNT_EXIST,
+			   _("Target doesn't exist"));
+		return;
+	}
+
+	/* Create the update xml */
+	doc = xml_new_change_doc (project);
+	if (!xml_write_remove_target (project, doc, g_node)) {
+		error_set (error, GBF_PROJECT_ERROR_DOESNT_EXIST,
+			   _("Target couldn't be removed"));
+		xmlFreeDoc (doc);
+		return;
+	}
+
+	DEBUG ({
+		xmlSetDocCompressMode (doc, 0);
+		xmlSaveFile ("/tmp/remove-target.xml", doc);
+	});
+
+	/* Update the project */
+	if (!project_update (project, doc, &change_set, error)) {
+		error_set (error, GBF_PROJECT_ERROR_PROJECT_MALFORMED,
+			   _("Unable to update project"));
+	}
+	xmlFreeDoc (doc);
+	change_set_destroy (change_set);
 }
 
 static const gchar * 
@@ -2878,6 +3335,10 @@ impl_name_for_type (GbfProject  *_project,
 		return _("Script");
 	} else if (!strcmp (type, "info")) {
 		return _("Info Documentation");
+	} else if (!strcmp (type, "java")) {
+		return _("Java Module");
+	} else if (!strcmp (type, "python")) {
+		return _("Python Module");
 	} else {
 		return _("Unknown");
 	}
@@ -2901,6 +3362,10 @@ impl_mimetype_for_type (GbfProject  *_project,
 		return "text/x-sh";
 	} else if (!strcmp (type, "info")) {
 		return "application/x-tex-info";
+	} else if (!strcmp (type, "java")) {
+		return "application/x-java";
+	} else if (!strcmp (type, "python")) {
+		return "application/x-python";
 	} else {
 		return "text/plain";
 	}
@@ -2910,7 +3375,7 @@ static gchar **
 impl_get_types (GbfProject *_project)
 {
 	return g_strsplit ("program:shared_lib:static_lib:"
-			   "man:data:script:info", ":", 0);
+			   "man:data:script:info:java:python", ":", 0);
 }
 
 static GbfProjectTargetSource * 
@@ -3127,7 +3592,7 @@ impl_remove_source (GbfProject  *_project,
 	doc = xml_new_change_doc (project);
 	if (!xml_write_remove_source (project, doc, g_node)) {
 		error_set (error, GBF_PROJECT_ERROR_DOESNT_EXIST,
-			   _("Source coudn't be removed"));
+			   _("Source couldn't be removed"));
 		xmlFreeDoc (doc);
 		return;
 	}
@@ -3147,6 +3612,22 @@ impl_remove_source (GbfProject  *_project,
 	xmlFreeDoc (doc);
 }
 
+static GtkWidget *
+impl_configure (GbfProject *_project, GError **error)
+{
+	GtkWidget *wid = NULL;
+	GError *err = NULL;
+	
+	g_return_val_if_fail (GBF_IS_PROJECT (_project), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+	
+	wid = gbf_am_properties_get_widget (GBF_AM_PROJECT (_project), &err);
+	if (err) {
+		g_propagate_error (error, err);
+	}
+	return wid;
+}
+
 static void
 gbf_am_project_class_init (GbfAmProjectClass *klass)
 {
@@ -3173,6 +3654,7 @@ gbf_am_project_class_init (GbfAmProjectClass *klass)
 	project_class->add_group = impl_add_group;
 	project_class->remove_group = impl_remove_group;
 	project_class->get_group = impl_get_group;
+	project_class->get_all_groups = impl_get_all_groups;
 	project_class->configure_group = impl_configure_group;
 	project_class->configure_new_group = impl_configure_new_group;
 
@@ -3190,6 +3672,7 @@ gbf_am_project_class_init (GbfAmProjectClass *klass)
 	project_class->configure_source = impl_configure_source;
 	project_class->configure_new_source = impl_configure_new_source;
 
+	project_class->configure = impl_configure;
 	project_class->name_for_type = impl_name_for_type;
 	project_class->mimetype_for_type = impl_mimetype_for_type;
 	project_class->get_types = impl_get_types;
