@@ -624,7 +624,18 @@ variable_value_edited_cb (GtkCellRendererText *cellrenderertext, gchar *arg1,
 	gtk_tree_model_get (model, &iter, COL_VAR_NAME, &variable_name, -1);
 
 	value = gbf_am_config_mapping_lookup (config, "variables");
-	variables = gbf_am_config_value_get_mapping (value);
+	if (!value)
+	{
+		variables = gbf_am_config_mapping_new ();
+		value = gbf_am_config_value_new (GBF_AM_TYPE_MAPPING);
+		gbf_am_config_value_set_mapping (value, variables);
+		gbf_am_config_mapping_insert (config, "variables", value);
+					      
+	}
+	else
+	{
+		variables = gbf_am_config_value_get_mapping (value);
+	}
 	value = gbf_am_config_mapping_lookup (variables, variable_name);
 
 	if (!value)
